/****************************************************************************
    Constant definition
*****************************************************************************/
#define ST_MAX_WORD_LEN  40
#define ST_DISPLAY_LEN   15

#define ST_MAX_SUG_WORDS 12
#define ST_GET_ALTERNATES 0x1

#define ST_INTERACTIVE   0x1
#define ST_BEEP          0x2

#define STFLAG_USE_APOSTROPHE 0x1      // treat apostrophe as a regular character
#define STFLAG_SPANISH_DLG    0x2      // spanish dialog box
#define STFLAG_APOSTROPHE_JOINER 0x4   // treat apostrophe as a word delimiter
#define STFLAG_NO_NUM_IN_WORD 0x8      // no number in word allowed
#define STFLAG_DUTCH_DLG      0x10     // Dutch diloag box
#define STFLAG_GERMAN_DLG     0x20     // German diloag box
#define STFLAG_FRENCH_DLG     0x40     // French diloag box
#define STFLAG_ALL_CAPS_TO_LOWER 0x80  // if word is all in caps, make it lower case - this is useful when using a case-sensitive dictionary
#define STFLAG_CHECK_LOWER_CASE  0x100 // if uppercase not found, then check for lowercase - userful when using the4 case-sensitive dictionary
#define STFLAG_RETURN_MSG_ID     0x200   // treat apostrophe as a regular character

// Error messages
#define MSG_FILE_NOT_FOUND     1
#define MSG_NO_EDITOR_INSTANCE 2
#define MSG_READ_ERROR         3
#define MSG_WRITE_ERROR        4
#define MSG_MEMORY_LOCK        5
#define MSG_NO_FILE_SIGNATURE  6
#define MSG_BAD_RTF_FILE       7
#define MSG_BAD_HTML_FILE      8
#define MSG_EXPIRED            9
#define MSG_PDF_INIT_ERROR    10
#define MSG_WEB               11

#define MSG_OTHER             99

/****************************************************************************
    Return codes from the 'SpellDict' routine.
****************************************************************************/
#define STD_FOUND         1
#define STD_NOT_FOUND     0
#define STD_ERROR         -1

/******************************************************************************
    WIN32 specific defines
*******************************************************************************/
#if defined (_WIN32)
   #if !defined(WIN32)
     #define WIN32
   #endif
#endif
#define SPELL32
#if defined (SPELL32)
   #define _export
   #undef  FreeProcInstance
   #define FreeProcInstance(x)
   #define GET_WM_COMMAND_ID(wp,lp) LOWORD(wp)
   #define CONTROL_ID(wp,lp)        LOWORD(wp)
   #define NOTIFY_MSG(wp,lp)        HIWORD(wp)
   #undef  farmalloc
   #define farmalloc  malloc
   #undef  farrealloc
   #define farrealloc realloc
   #undef  farfree
   #define farfree    free
   #define _fmemchr   memchr
#else
   #define GET_WM_COMMAND_ID(wp,lp) (wp)
   #define CONTROL_ID(wp,lp)        (wp)
   #define NOTIFY_MSG(wp,lp)        HIWORD(lp)
#endif

// win32 to win64 contant declation to provide for common source code
#if !defined(WIN64)
  #if !defined(GWLP_WNDPROC)
    #define GWLP_WNDPROC  GWL_WNDPROC
  #endif

  #if !defined(DWLP_USER)
     #define DWLP_USER     DWL_USER
  #endif
#endif


/****************************************************************************
   Instructions for packing the following structure
*****************************************************************************/
#ifndef RC_INVOKED
   #pragma pack(1)
#endif

/******************************************************************************
    structure to return the result from the 'SpellWord' routine
*******************************************************************************/
struct StrStResult {
       UINT code;                        // result codes
       char replace[ST_MAX_WORD_LEN+1];  // word to substitute for the current word
       char reserved[1];                 // pad to 4 byte boundary - needed by VB 32 bit
       int  TotalAltWords;               // number of alternate words
       char AltWord[ST_MAX_SUG_WORDS][ST_MAX_WORD_LEN+1];  // each alternate word
       };

/****************************************************************************
   Instructions to resume the default structure packing.
*****************************************************************************/
#ifndef RC_INVOKED
   #pragma pack()
#endif

/******************************************************************************
    result codes from the SpellWord routine (variable 'code' in structure
    StrStResult).  To check for the specific condiction, use the AND (&)
    operator.
*******************************************************************************/

#define ST_IGNORE        0x1
#define ST_REPLACE       0x2
#define ST_REFRESH       0x4
#define ST_TOO_LONG      0x8
#define ST_ERROR         0x10
#define ST_EXIT          0x20
#define ST_ADD           0x40
#define ST_INPUT         0x80
#define ST_ALL           0x100

/*****************************************************************************
    Prototypes for the Spell Time DLL functions.
   When calling SPELL from a C program, leave the SPL_CPLUSPLUS symbol undefined.

   When calling SPELLL from a C++ program, define the SPL_CPLUSPLUS as following:

   #define SPL_CPLUSPLUS extern "C"
   #include "spell.h"
*****************************************************************************/
#ifndef SPL_CPLUSPLUS
   #if defined(__cplusplus)
      #define SPL_CPLUSPLUS extern "C"
   #else
      #define SPL_CPLUSPLUS
   #endif
#endif

SPL_CPLUSPLUS int  WINAPI _export SpellDict(LPSTR,int,UINT);
SPL_CPLUSPLUS int  WINAPI _export SpellWord(LPSTR,UINT,HWND,struct StrStResult far *);
SPL_CPLUSPLUS int  WINAPI _export StResetUserDict(LPSTR,LPSTR);
SPL_CPLUSPLUS int  WINAPI _export StParseLine(LPSTR,LPSTR,LPINT,LPINT,int);
SPL_CPLUSPLUS int  WINAPI _export StClearHist(void);
SPL_CPLUSPLUS int  WINAPI _export StDelUserWord(LPSTR);
SPL_CPLUSPLUS int  WINAPI _export StLoadResult(struct StrStResult far *);
SPL_CPLUSPLUS int  WINAPI _export ToSpellHist(LPSTR,char,LPSTR);
SPL_CPLUSPLUS int  WINAPI _export ToUserDict(LPSTR);
SPL_CPLUSPLUS UINT WINAPI _export StGetReplacement(LPSTR);
SPL_CPLUSPLUS int  WINAPI _export StGetAlternateWord(int, LPSTR);
SPL_CPLUSPLUS BOOL WINAPI _export StSetDictName(LPSTR);
SPL_CPLUSPLUS DWORD WINAPI _export StSetFlags(BOOL,DWORD);
SPL_CPLUSPLUS BOOL WINAPI _export StAddVowel(BYTE);
SPL_CPLUSPLUS int  WINAPI _export SpellString(LPSTR,LPSTR,int,HWND);
SPL_CPLUSPLUS int WINAPI _export  StStrLwrEx(LPBYTE);
SPL_CPLUSPLUS BOOL  WINAPI _export StGetInitialized(void);

SPL_CPLUSPLUS int  WINAPI _export SpellDict2(DWORD_PTR,LPSTR,int,UINT);
SPL_CPLUSPLUS int  WINAPI _export SpellWord2(DWORD_PTR,LPSTR,UINT,HWND,struct StrStResult far *);
SPL_CPLUSPLUS int  WINAPI _export StResetUserDict2(DWORD_PTR,LPSTR,LPSTR);
SPL_CPLUSPLUS int  WINAPI _export StParseLine2(DWORD_PTR,LPSTR,LPSTR,LPINT,LPINT,int);
SPL_CPLUSPLUS int  WINAPI _export StClearHist2(DWORD_PTR);
SPL_CPLUSPLUS int  WINAPI _export StDelUserWord2(DWORD_PTR,LPSTR);
SPL_CPLUSPLUS int  WINAPI _export StLoadResult2(DWORD_PTR,struct StrStResult far *);
SPL_CPLUSPLUS int  WINAPI _export ToSpellHist2(DWORD_PTR,LPSTR,char,LPSTR);
SPL_CPLUSPLUS int  WINAPI _export ToUserDict2(DWORD_PTR,LPSTR);
SPL_CPLUSPLUS UINT WINAPI _export StGetReplacement2(DWORD_PTR,LPSTR);
SPL_CPLUSPLUS int  WINAPI _export StGetAlternateWord2(DWORD_PTR,int, LPSTR);
SPL_CPLUSPLUS BOOL WINAPI _export StSetDictName2(DWORD_PTR,LPSTR);
SPL_CPLUSPLUS DWORD WINAPI _export StSetFlags2(DWORD_PTR,BOOL,DWORD);
SPL_CPLUSPLUS DWORD_PTR WINAPI _export StInitSession(void);
SPL_CPLUSPLUS BOOL WINAPI  _export StEndSession(DWORD_PTR);
SPL_CPLUSPLUS BOOL WINAPI _export StAddVowel2(DWORD_PTR,BYTE);
SPL_CPLUSPLUS int  WINAPI _export SpellString2(DWORD_PTR,LPSTR,LPSTR,int,HWND);
SPL_CPLUSPLUS BOOL WINAPI _export StSetLicenseKey(LPBYTE);
SPL_CPLUSPLUS BOOL  WINAPI _export StGetInitialized2(DWORD_PTR);

// Undocumented functions - do not use them without consultation with Sub Systems, Inc.
SPL_CPLUSPLUS BOOL WINAPI _export StSetDialogPos(int,int);
SPL_CPLUSPLUS BOOL WINAPI _export StSetDialogPos2(DWORD_PTR,int,int);
SPL_CPLUSPLUS BOOL WINAPI _export StInEval();

