/*==============================================================================
   TER_SPL.C
   TER Spell Checker interface.

   SpellTime &
   TER Editor Developer's Kit    
   Sub Systems, Inc.
   Software License Agreement  (1991)              
   ----------------------------------
   This license agreement allows the purchaser the right to modify the 
   source code to incorporate in an application larger than the editor itself.
   The target application must not be a programmer's utility 'like' a text editor.
   Sub Systems, Inc. reserves the right to prosecute anybody found to be 
   making illegal copies of the executable software or compiling the source
   code for the purpose of selling there after.
===============================================================================*/

#define  PREFIX extern
#include "ter_hdr.h"

#include "spell.h"

/*****************************************************************************
    Global variables specific to this module only
******************************************************************************/
static int  WordIndex,                 /* position of the current word in the current line */
            WordLen;                   /* current word length */

/*****************************************************************************
     Function prototypes
*****************************************************************************/
int     ReplaceWord(PTERWND,char far *,int,int far *,char far *);
int     SpellCheck(PTERWND);
void    HilightWord(PTERWND);

/******************************************************************************
    SpellCheck:
     1. Determine the range of lines to spell check.
     2. Spell check each line in the specified range.  The misspelled words
        are highlighted and the user is allowed to provide a correction.
******************************************************************************/
SpellCheck(PTERWND w)
{
     long BegRow,EndRow;
     int  BegCol,EndCol;
     int  LineIndex;
     LPSTR ptr;
     char  string[80];
     long TotalWords=0,WrongWords=0;
     char CurWord[ST_MAX_WORD_LEN+1];
     HCURSOR hSaveCursor;
     struct StrStResult StResult;        /* result from 'SpellWord' */


    /************* determine the range of the text to spell check ***********/
    if (HilightType==HILIGHT_LINE) {         /* line block */
        NormalizeBlock(w);                   /* normalize beginning and ending of the block */
        BegRow=HilightBegRow;
        BegCol=0;
        EndRow=HilightEndRow;
        EndCol=LineLen[EndRow];
        if (EndCol<0) EndCol=0;
        HilightType=HILIGHT_OFF;             /* reset the line block */
        StretchHilight=FALSE;
    }
    else if (HilightType==HILIGHT_CHAR) {  /* character block */
        NormalizeBlock(w);                   /* normalize beginning and ending of the block */
        BegRow=HilightBegRow;
        BegCol=HilightBegCol;
        if (BegCol>=LineLen[BegRow]) BegCol=LineLen[BegRow]-1;
        if (BegCol<0) BegCol=0;
        
        EndRow=HilightEndRow;
        EndCol=HilightEndCol-1;
        if (EndCol>=LineLen[EndRow]) EndCol=LineLen[EndRow]-1;
        if (EndCol<0) EndCol=0;
        if (BegRow==EndRow && BegCol>EndCol) BegCol=EndCol;
        HilightType=HILIGHT_OFF;             /* reset the line block */
        StretchHilight=FALSE;
    }
    else if (HilightType!=HILIGHT_OFF)  {   /* column block not allowed (TE 3.0) */
        MessageBox(NULL,"Column Block Not Allowed for Spell Checking",NULL,MB_OK);
        return TRUE;
    }                                   
    else {                                  /* if text not highlighted, check the entire file */
        BegRow=0;
        BegCol=0;
        EndRow=TotalLines-1;
        if (EndRow<0) EndRow=0;
        EndCol=LineLen[EndRow];
        if (EndCol<0) EndCol=0;
    }

    hSaveCursor=SetCursor(hWaitCursor);   /* show hour glass while reading */
    StClearHist();                        /* clear the SpellTime history buffer */

    for (CurLine=BegRow;CurLine<=EndRow;CurLine++) {         /* spell check each line */
        if (LineLen[CurLine]==0) continue;
        
        ptr=sLock(hLine[CurLine]);        /* lock the current line */
        
        LineIndex=0;                      /* number of characters checked in the current line */
        
        while ((WordLen=StParseLine(ptr,CurWord,&WordIndex,&LineIndex,LineLen[CurLine]))>0) {  /* process each word */
           if (CurLine==BegRow && WordIndex<BegCol) continue;
           if (CurLine==EndRow && WordIndex>EndCol) continue;

           /* Check once without user interaction, and if word is found
              misspelled, then call the 'SpellWord' function again with
              ST_INTERACTIVE flag */

           if (!SpellWord(CurWord,0,hTerWnd,&StResult)) {  /* mis-spelled word */
              if (StResult.code&ST_ERROR) goto END;
              
              SetCursor(hSaveCursor);                      /* display regular cursor */
              HilightWord(w);                              /* highlight the misspelled word */
              SpellWord(CurWord,ST_INTERACTIVE,hTerWnd,&StResult);    
              
              if (StResult.code&(ST_IGNORE|ST_REPLACE|ST_ADD|ST_INPUT)) {
                  WrongWords++;
              }
              else if (StResult.code&(ST_EXIT|ST_ERROR)) goto END;
              
              if (StResult.replace[0]) {
                 sUnlock(hLine[CurLine]);            /* unlock the current line */
                 ReplaceWord(w,CurWord,WordIndex,&LineIndex,StResult.replace);
                 ptr=sLock(hLine[CurLine]);          /* lock again */
              }
              SetCursor(hWaitCursor);   /* resume */
           }    
           TotalWords++;
        } 
        sUnlock(hLine[CurLine]);
    }
    CurLine=EndRow;

    END:
    /* update the user dictionary file */
    StResetUserDict(NULL,NULL);
    
    SetCursor(hSaveCursor);
    HilightType=HILIGHT_OFF;
    
    if ((CurLine-BeginLine)>=(WinHeight-1) || (CurLine-BeginLine)<0) {
       BeginLine=CurLine-(WinHeight)/2;    /* position the current toward the center */
       if (BeginLine<0) BeginLine=0;
    }
    CurRow=CurLine-BeginLine;
    CurCol=0;
    PaintFlag=PAINT_WIN;
    PaintTer(w);
    
    wsprintf(string,"Words Checked: %ld,  Incorrect Words: %ld",TotalWords,WrongWords);
    MessageBox(hTerWnd,string,"Spell Session Over",MB_OK);

    return TRUE;
}

/******************************************************************************
    HilightWord:
    This routine highlight the misspelled word.  The word location is given
    by these variables: CurLine -> Current line number, WordIndex -> word position
    within the current line, WordLen -> length of the misspelled word.
******************************************************************************/
void HilightWord(PTERWND w)
{
    HilightType=HILIGHT_CHAR;             /* highlight the misspelled word */
    HilightBegRow=HilightEndRow=CurLine;  /* highlight the misspelled word */
    CurCol=HilightBegCol=WordIndex;
    HilightEndCol=WordIndex+WordLen;

    if ((CurLine-BeginLine)>WinHeight/2 || (CurLine-BeginLine)<0) {
       BeginLine=CurLine-(WinHeight)/4;   /* position toward top */
       if (BeginLine<0) BeginLine=0;
    }
    
    CurRow=CurLine-BeginLine;
    WrapFlag=WRAP_OFF;                    /* refresh from the text */
    PaintTer(w);     
}

/******************************************************************************
    ReplaceWord:
    This routine replaces an misspelled word from the current line by a replacement
    word.  The line length and current line pointer variables
    are adjusted accordingly.  
******************************************************************************/
ReplaceWord(PTERWND w,char far *CurWord,int WordIndex,int far *LineIndex,char far *NewWord)
{
   int OldLen,NewLen,AdjLen,i;
   LPSTR ptr;


   OldLen=lstrlen(CurWord);
   NewLen=lstrlen(NewWord);
   AdjLen=NewLen-OldLen;

   if (AdjLen!=0) {                    /* make necessary adjustments */
      if ((LineLen[CurLine]+AdjLen)>(LineWidth-1)) return FALSE;
      *LineIndex=(*LineIndex)+AdjLen;  /* adjust the current word pointer within the current line*/
   }
   
   if (AdjLen<0) MoveLineData(w,CurLine,WordIndex,abs(AdjLen),'D'); /* delete character */
   if (AdjLen>0) MoveLineData(w,CurLine,WordIndex,abs(AdjLen),'B'); /* insert spaces */

   ptr=sLock(hLine[CurLine]);
   for(i=0;i<NewLen;i++) ptr[WordIndex+i]=NewWord[i];    /* substitute the new word */
   sUnlock(hLine[CurLine]);

   TerArg.modified++;

   return TRUE;
}

